" Specify a directory for plugins (for Neovim: ~/.local/share/nvim/plugged)
call plug#begin('~/.config/nvim/plugged')

" ----------------------------- Plugins --------------------------------------
" Automatically match structures as you type
Plug 'tpope/vim-endwise'
Plug 'jiangmiao/auto-pairs'
" Tab completion
Plug 'ervandew/supertab'
" EasyMotion
Plug 'easymotion/vim-easymotion'
Plug 'haya14busa/incsearch.vim'
Plug 'haya14busa/incsearch-easymotion.vim'
" Fuzzy search
Plug 'junegunn/fzf.vim'
Plug 'pbogut/fzf-mru.vim'
" Remove extraneous whitespace when edit mode is exited
Plug 'thirtythreeforty/lessspace.vim'
" Status bar mods
Plug 'vim-airline/vim-airline'
" Manage Project sessions
Plug 'thaerkh/vim-workspace'
" Commenter
Plug 'tpope/vim-commentary'
" Theme
Plug 'sickill/vim-monokai'
Plug 'justinmk/vim-syntax-extra'
Plug 'octol/vim-cpp-enhanced-highlight'
" Autoload and read from dish if file changes
Plug 'tmux-plugins/vim-tmux-focus-events'
Plug 'wellle/tmux-complete.vim'
" For LaTeX support
Plug 'lervag/vimtex'
" For git support
Plug 'airblade/vim-gitgutter'
Plug 'tpope/vim-fugitive'
Plug 'junegunn/gv.vim'
Plug 'rhysd/git-messenger.vim'
Plug 'christoomey/vim-conflicted'
Plug 'rbong/vim-flog'
Plug 'whiteinge/diffconflicts'
" For tmux yank
Plug 'vim-utils/vim-husk'
" GDB
Plug 'sakhnik/nvim-gdb', { 'do': ':UpdateRemotePlugins' }
" Lisp
Plug 'guns/vim-sexp', { 'for': [ 'scheme', 'lisp', 'clojure' ] }
Plug 'tpope/vim-sexp-mappings-for-regular-people', { 'for': [ 'scheme', 'lisp', 'clojure' ] }
Plug 'junegunn/rainbow_parentheses.vim', { 'for': [ 'scheme', 'lisp', 'clojure' ] }
Plug 'kovisoft/slimv', { 'for': [ 'scheme', 'lisp', 'clojure' ] }
" Haskell
Plug 'ekmett/haskell-vim', { 'for': [ 'haskell', 'cabal' ] }
Plug 'ndmitchell/ghcid', { 'rtp': 'plugins/nvim' }
Plug 'Twinside/vim-hoogle', { 'for': 'haskell' }
Plug 'hspec/hspec.vim'
Plug 'pbrisbin/vim-syntax-shakespeare'
Plug 'zenzike/vim-haskell-unicode', { 'for': 'haskell' }
" For Nix
Plug 'LnL7/vim-nix', { 'for': 'nix' }
" For autocompletion
Plug 'Shougo/deoplete.nvim', { 'do': ':UpdateRemotePlugins' }
Plug 'deoplete-plugins/deoplete-tag'
" For Python
Plug 'davidhalter/jedi-vim', { 'for': 'python' }
Plug 'zchee/deoplete-jedi', { 'for': 'python' }
Plug 'numirias/semshi', { 'for': 'python', 'do': ':UpdateRemotePlugins' }
" Autocomplete for Rust
Plug 'sebastianmarkow/deoplete-rust', { 'for': 'rust' }
Plug 'cespare/vim-toml', { 'for': 'rust' }
Plug 'rust-lang/rust.vim', { 'for': 'rust' }
Plug 'mhinz/vim-crates'
" Erlang Support
Plug 'vim-erlang/vim-erlang-tags', { 'for': 'erlang' }
Plug 'vim-erlang/vim-erlang-runtime', { 'for': 'erlang' }
Plug 'vim-erlang/vim-erlang-omnicomplete', { 'for': 'erlang' }
Plug 'vim-erlang/vim-erlang-compiler', { 'for': 'erlang' }
" Purescript
Plug 'purescript-contrib/purescript-vim', { 'for': 'purescript' }
Plug 'frigoeu/psc-ide-vim', { 'for': 'purescript' }
" Neo/Async stuff
Plug 'sbdchd/neoformat'
Plug 'neomake/neomake'
" Quickfix
Plug 'sk1418/QFGrep'
Plug 'ronakg/quickr-cscope.vim'
Plug 'stefandtw/quickfix-reflector.vim'
" Absolute essentials miscellaneous
Plug 'tpope/vim-surround'
Plug 'tpope/vim-repeat'
Plug 'tpope/vim-eunuch'
Plug 'tpope/vim-sleuth'
Plug 'tpope/vim-vinegar'
Plug 'wellle/targets.vim'
Plug 'Yggdroot/indentLine'
" Miscellaneous
Plug 'editorconfig/editorconfig-vim'
Plug 'igankevich/mesonic'
Plug 'psliwka/vim-smoothie'
Plug 'dag/vim-fish'
Plug 'lifepillar/pgsql.vim'
Plug 'simnalamburt/vim-mundo'
Plug 'milkypostman/vim-togglelist'
Plug 'liuchengxu/vim-which-key'
Plug 'godlygeek/tabular'

" Initialize plugin system
call plug#end()

" ----------------------------- Settings -------------------------------------
" Set up leaders
let mapleader="\<SPACE>"
let maplocalleader=","

set colorcolumn=80      " Highlight 80th column
set laststatus=2        " Always show status bar
set updatetime=300      " Let plugins show effects after 500ms
set mouse-=a            " Disable mouse click to go to position
set encoding=utf-8
set exrc                " Allow loading local .nvimrc files
set secure              " Disallow use of autocmd, shell and write in local rc
set showcmd             " Show (partial) command in status line.
set showmatch           " Show matching brackets.
set ruler               " Show the line and column numbers of the cursor.
set number              " Show the line numbers on the left side.
set formatoptions+=o    " Continue comment marker in new lines.
set formatoptions+=j    " Delete comment character when joining commented lines
set textwidth=78        " Hard-wrap long lines as you type them.
set softtabstop=4       " Finetunes the amount of white space to be added.
set tabstop=4           " Render TABs using this many spaces.
set shiftwidth=4        " Indentation amount for < and > commands.
set smarttab            " Indent with tabs, align with spaces.
set expandtab           " When on, uses space instead of tabs.
set autoindent          " Copy indent from current line when starting a new line
set noerrorbells        " No beeps.
set modeline            " Enable modeline.
set linespace=0         " Set line-spacing to minimum.
set nojoinspaces        " Prevents inserting two spaces after punctuation on a join (J)
set noshowmode          " Because of neovim's cursor shape
set splitbelow          " Horizontal split below current.
set splitright          " Vertical split to right of current.
set nobackup            " no backup files
set nowritebackup       " only in case you don't want a backup file while editing
set noswapfile          " no swap files
set foldmethod=syntax   " Create folds based on files syntax
set nofoldenable        " Open folds by default
set undofile            " Enable undo persistence across sessions

" Wild menu
set wildmenu
set wildmode=list:longest,full
set wildoptions=pum
set pumblend=30
set wildignore+=.hg,.git,.svn                    " Version control
set wildignore+=*.aux,*.out,*.toc                " LaTeX intermediate files
set wildignore+=*.jpg,*.bmp,*.gif,*.png,*.jpeg   " binary images
set wildignore+=*.o                              " compiled object files
set wildignore+=*.sw?                            " Vim swap files
set wildignore+=*.luac                           " Lua byte code
set wildignore+=*.pyc                            " Python byte code
set wildignore+=*.orig                           " Merge resolution files

if !&scrolloff
  set scrolloff=3       " Show next 3 lines while scrolling.
endif
if !&sidescrolloff
  set sidescrolloff=5   " Show next 5 columns while side-scrolling.
endif
set nostartofline       " Do not jump to first character with page commands.

set ignorecase          " Make searching case insensitive
set smartcase           " ... unless the query has capital letters.
set magic               " Use 'magic' patterns (extended regular expressions).
set autoread            " Autoload file if it changes on disk
set termguicolors       " Enable colors for terminal
set fileformat=unix
set inccommand=split

set clipboard^=unnamed,unnamedplus
set backspace=indent,eol,start
set completeopt=menu,noselect,preview,noinsert
" Required for vim-workspace
" See https://github.com/thaerkh/vim-workspace/issues/11
set sessionoptions-=blank

" Theme
colorscheme monokai
set background=dark
" Fixes colors in Neovim terminal
highlight LineNr ctermbg=000

" --------------------------- Key Bindings -----------------------------------
" Use <C-L> to clear the highlighting of :set hlsearch.
if maparg('<C-L>', 'n') ==# ''
  nnoremap <silent> <C-L> :nohlsearch<CR><C-L>
endif

" Remap escape keys to something usable on home row
inoremap jk <Esc>
cnoremap jk <C-C>
inoremap <Esc> <Nop>
cnoremap <Esc> <Nop>
" Use ; for commands.
nnoremap ; :
" Use Q to execute default register.
nnoremap Q <Nop>

" For Fuzzy
nnoremap ff  :GFiles<CR>
nnoremap f?  :GFiles?<CR>
nnoremap fF  :Files<CR>
nnoremap fb  :Buffers<CR>
nnoremap fL  :Lines<CR>
nnoremap fl  :BLines<CR>
nnoremap ft  :BTags<CR>
nnoremap fT  :Tags<CR>
nnoremap fc  :BCommits<CR>
nnoremap fC  :Commits<CR>
nnoremap fh  :History:<CR>
nnoremap fH  :History/<CR>
nnoremap fm  :Commands<CR>
nnoremap fo  :Locate<SPACE>
nnoremap fk  :Maps<CR>
nnoremap fr  :FZFMru<CR>
nnoremap f/  :Rg<CR>
nnoremap fs  :exe ':Rg ' . expand('<cword>')<CR>
imap <C-x><C-w> <plug>(fzf-complete-word)
imap <C-x><C-p> <plug>(fzf-complete-path)
imap <C-x><C-f> <plug>(fzf-complete-file)
imap <C-x><C-l> <plug>(fzf-complete-line)
" Undo tree
nnoremap <Leader>u :MundoToggle<CR>
" Save
nnoremap <Leader>w <Esc>:w<CR>
" Search and Replace
nnoremap sr :%s//g<Left><Left>
" Manage window splitting
nnoremap sh  <Esc>:sp<CR>
nnoremap sv  <Esc>:vsp<CR>
" Quit
nnoremap <Leader>x  <Esc>:x<CR>
nnoremap <Leader>q  <Esc>:q<CR>
nnoremap <Leader>Q  <Esc>:qa<CR>
" Navigate buffers
nnoremap [b :bprevious<CR>
nnoremap ]b :bnext<CR>
nnoremap [B :bfirst<CR>
nnoremap ]B :blast<CR>
" Terminal
nnoremap <Leader>te :vsp<CR> :terminal<CR>
" For git
nnoremap <Leader>gm :GitMessenger<CR>
nnoremap <Leader>gl :0Glog<CR>
nnoremap <Leader>gL :Glog<CR>
nnoremap <Leader>gd :Gdiff<CR>
nnoremap <Leader>gD :Gdiffsplit<CR>
nnoremap <Leader>gs :Gstatus<CR>
nnoremap <Leader>gc :Gcommit -v -q --signoff<CR>
nnoremap <Leader>gt :Gcommit -v -q --signoff %:p<CR>
nnoremap <Leader>gp :Git push<CR>
nnoremap <Leader>gu :Git push -u<SPACE>
nnoremap <Leader>gr :Git remote -v<CR>
nnoremap <Leader>gb :Git branch<SPACE>
nnoremap <Leader>go :Git checkout<SPACE>
nnoremap <Leader>g- :Git stash<CR>:e<CR>
nnoremap <Leader>g+ :Git stash pop<CR>:e<CR>
nnoremap <Leader>gf :exe ':Ggrep ' . expand('<cword>')<CR>
nnoremap <Leader>gF :Ggrep<SPACE>
nnoremap <Leader>gg :GGrep<SPACE>
" For gitgutter
nnoremap ]h :GitGutterNextHunk<CR>
nnoremap [h :GitGutterPrevHunk<CR>
nnoremap us :GitGutterStageHunk<CR>
nnoremap ud :GitGutterUndoHunk<CR>
nnoremap ur :GitGutterPreviewHunk<CR>
nnoremap uq :GitGutterQuickFix<CR>
nnoremap uf :GitGutterFold<CR>
" For Neomake/Neoformat
nnoremap nm  :Neomake<CR>
nnoremap nc  :NeomakeClean<CR>
nnoremap ns  :NeomakeSh<SPACE>
nnoremap njl :NeomakeListJobs<CR>
nnoremap nja :NeomakeCancelJobs<CR>
nnoremap njc :NeomakeCancelJob<SPACE>
nnoremap nf  :Neoformat<CR>
" For workspace
nnoremap <Leader>ws  :ToggleWorkspace<CR>
" Quickfix & Location list mappings
nnoremap qo :copen<CR>
nnoremap qc :cclose<CR>
nnoremap [q :cprevious<CR>
nnoremap ]q :cnext<CR>
nnoremap [Q :cfirst<CR>
nnoremap ]Q :clast<CR>
nnoremap qs :vimgrep<SPACE>
nnoremap qt :call ToggleQuickfixList()<CR>
nnoremap Lo :lopen<CR>
nnoremap Lc :lclose<CR>
nnoremap [l :lprevious<CR>
nnoremap ]l :lnext<CR>
nnoremap [L :lfirst<CR>
nnoremap ]L :llast<CR>
nnoremap Ls :lvimgrep<SPACE>
nnoremap Lt :call ToggleLocationList()<CR>

" Key Bindings to help with terminal mode
:tnoremap jk <C-\><C-n>
" Key bindings to move between window splits
nnoremap gh <C-w>h
nnoremap gj <C-w>j
nnoremap gk <C-w>k
nnoremap gl <C-w>l

" Disable Arrow Keys
inoremap  <Up>     <NOP>
inoremap  <Down>   <NOP>
inoremap  <Left>   <NOP>
inoremap  <Right>  <NOP>
noremap   <Up>     <NOP>
noremap   <Down>   <NOP>
noremap   <Left>   <NOP>
noremap   <Right>  <NOP>

" Whick key
nnoremap <silent> <Leader> :<C-U>WhichKey '<Space>'<CR>
nnoremap <silent> <LocalLeader> :<C-U>WhichKey ','<CR>

" Bindings for easy motion
" Character motions
map  <Leader><Leader>c <Plug>(easymotion-bd-f)
nmap <Leader><Leader>c <Plug>(easymotion-overwin-f)
" Line motions
map <Leader><Leader>L <Plug>(easymotion-bd-jk)
nmap <Leader><Leader>L <Plug>(easymotion-overwin-line)
" Word motions
map  <Leader><Leader>w <Plug>(easymotion-bd-w)
nmap <Leader><Leader>w <Plug>(easymotion-overwin-w)
" Line motion jump
map <Leader>l <Plug>(easymotion-lineforward)
map <Leader>j <Plug>(easymotion-j)
map <Leader>k <Plug>(easymotion-k)
map <Leader>h <Plug>(easymotion-linebackward)
" Incremental search + easymotion
noremap <silent><expr> /  incsearch#go(<SID>incsearch_config())
noremap <silent><expr> ?  incsearch#go(<SID>incsearch_config({'command': '?'}))
noremap <silent><expr> g/ incsearch#go(<SID>incsearch_config({'is_stay': 1}))

" Tag helpers
nnoremap gd :exec("tag ".expand("<cword>"))<CR>
nnoremap <C-\> :vsp <CR>:exec("tag ".expand("<cword>"))<CR>
nnoremap <A-]> :sp <CR>:exec("tag ".expand("<cword>"))<CR>

" Tabular helpers
vmap a= :Tabularize /=<CR>
vmap a; :Tabularize /::<CR>
vmap a- :Tabularize /-><CR>
vmap a: :Tabularize /:r1c1l0<CR>

command! -bang -nargs=* GGrep
  \ call fzf#vim#grep(
  \   'git grep --line-number '.shellescape(<q-args>), 0,
  \   fzf#vim#with_preview({'dir': systemlist('git rev-parse --show-toplevel')[0]}), <bang>0)

" --------------------------- Autocmd groups ---------------------------------
augroup vimplug_maps
    au!
    au FileType vim nmap pg :PlugUpgrade<CR>
    au FileType vim nmap pd :PlugUpdate<CR>
    au FileType vim nmap pc :PlugClean<CR>
	au FileType vim nmap pr :so %<CR>
augroup END

augroup haskell_maps
    au!
	au FileType haskell nmap <LocalLeader>t  :NeomakeSh stack exec -- fast-tags -R .<CR>
	au FileType haskell nmap <LocalLeader>g  :Ghcid<CR>
	au FileType haskell nmap <LocalLeader>k  :GhcidKill<CR>
	au FileType haskell nmap <LocalLeader>c  :HoogleClose<CR>
	au FileType haskell nmap <LocalLeader>o  :exe ':Hoogle ' . expand('<cword>')<CR>
	au FileType haskell nmap <LocalLeader>i  :exe ':HoogleInfo ' . expand('<cword>')<CR>
	au FileType haskell setlocal keywordprg=:Hoogle
augroup END

augroup purescript_maps
	au!
	au FileType purescript nmap <buffer> <silent> <LocalLeader>L :Plist<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>l :Pload<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>r :Prebuild<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>f :PaddClause<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>T :PaddType<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>a :Papply<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>c :Pcase<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>i :Pimport<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>q :PaddImportQualifications<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>g :Pgoto<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>p :Pursuit<CR>
	au FileType purescript nmap <buffer> <silent> <LocalLeader>t :Ptype<CR>
	au FileType purescript au BufWritePost <buffer> Prebuild
	au FileType purescript setlocal keywordprg=:Pursuit
augroup END

augroup c_maps
	au!
	au FileType c nmap <LocalLeader>ct :NeomakeSh ctags -R .<CR>
	au FileType c nmap <LocalLeader>cu :NeomakeSh cscope -bqR<CR>
	au FileType c nmap <LocalLeader>cr :cs reset<CR>

	au FileType c nmap <silent> <LocalLeader>s  <Plug>(quickr_cscope_symbols)
	au FileType c nmap <silent> <LocalLeader>g  <Plug>(quickr_cscope_global)
	au FileType c nmap <silent> <LocalLeader>h  <Plug>(quickr_cscope_global_split)
	au FileType c nmap <silent> <LocalLeader>v  <Plug>(quickr_cscope_global_vert_split)
	au FileType c nmap <silent> <LocalLeader>d  <Plug>(quickr_cscope_functions)
	au FileType c nmap <silent> <LocalLeader>c  <Plug>(quickr_cscope_callers)
	au FileType c nmap <silent> <LocalLeader>t  <Plug>(quickr_cscope_text)
	au FileType c nmap <silent> <LocalLeader>e  <Plug>(quickr_cscope_egrep)
	au FileType c nmap <silent> <LocalLeader>f  <Plug>(quickr_cscope_files)
	au FileType c nmap <silent> <LocalLeader>i  <Plug>(quickr_cscope_includes)
	au FileType c nmap <silent> <LocalLeader>a  <Plug>(quickr_cscope_assignments)
augroup END

augroup rust_maps
	au!
	au FileType rust nmap <buffer> gd <plug>DeopleteRustGoToDefinitionDefault
	au FileType rust nmap <buffer> K  <plug>DeopleteRustShowDocumentation
	au FileType rust nmap <buffer> gv <plug>DeopleteRustGoToDefinitionVSplit
	au FileType rust nmap <buffer> g- <plug>DeopleteRustGoToDefinitionSplit
	au FileType rust nmap <buffer> gt <plug>DeopleteRustGoToDefinitionTab
	" Taken from http://seenaburns.com/vim-setup-for-rust/
	" Neomake
	" Gross hack to stop Neomake running when exitting because it creates a zombie cargo check process
	" which holds the lock and never exits. But then, if you only have QuitPre, closing one pane will
	" disable neomake, so BufEnter reenables when you enter another buffer.
	let s:quitting = 0
	au FileType rust au QuitPre let s:quitting = 1
	au FileType rust au BufEnter let s:quitting = 0
	au FileType rust au BufWritePost if ! s:quitting | Neomake | else | echom "Neomake disabled" | endif
augroup END

augroup rainbow_lisp
	autocmd!
	autocmd FileType lisp,clojure,scheme RainbowParentheses
augroup END

" Toggles search highlighting off/on according to current mode. Source: http://blog.sanctum.geek.nz/vim-search-highlighting/
augroup toggle_search
	autocmd!
	autocmd InsertEnter * setlocal nohlsearch
	autocmd InsertLeave * setlocal hlsearch
augroup END

augroup quickfix
	au!
    " Close QF window if it is last window
	au WinEnter * if winnr('$') == 1 && &buftype == "quickfix"|q|endif
    " Existing bindings for QFGrep
    " <Leader>g input pattern to do further filtering
    " <Leader>v input pattern to do further inverted filtering
    " <Leader>r restore the Quickfix/location list with original entries
augroup END

augroup terminal_job
	au!
	au TermOpen * startinsert
	au TermOpen * setlocal listchars= nonumber norelativenumber
augroup END

augroup indentation_defaults
	au!
	au BufRead,BufNewFile *.c set noexpandtab
	au BufRead,BufNewFile *.h set noexpandtab
	au BufRead,BufNewFile Makefile* set noexpandtab
	au BufRead,BufNewFile *.vim setlocal noet ts=4 sw=4 sts=4
	au BufRead,BufNewFile *.txt setlocal noet ts=4 sw=4
	au BufRead,BufNewFile *.lua setlocal noet ts=4 sw=4 sts=4
	au BufRead,BufNewFile *.py setlocal ts=4 sts=4 sw=4 tw=80 smarttab et completeopt-=preview
	au BufRead,BufNewFile */gst-*/*.[ch] set et sw=2
	au BufRead,BufNewFile */gstreamer-*/*.[ch] set et sw=2
	au BufRead,BufNewFile */pulseaudio/*.[ch] set et sw=4 tw=128
	au BufRead,BufNewFile *.purs set et sw=2 sts=2 si
	au FileType fstab,systemd set noexpandtab
	au FileType gitconfig,sh,toml set noexpandtab
augroup END

augroup spell_check
	au!
	autocmd FileType gitcommit setlocal spell textwidth=72
	autocmd BufRead,BufNewFile *.md,*.txt setlocal spell
augroup END

" Automatically resize the window
autocmd VimResized * wincmd =

" --------------------------- Plugin settings --------------------------------
let g:haskell_enable_quantification = 1   " to enable highlighting of `forall`
let g:haskell_enable_recursivedo = 1      " to enable highlighting of `mdo` and `rec`
let g:haskell_enable_arrowsyntax = 1      " to enable highlighting of `proc`
let g:haskell_enable_pattern_synonyms = 1 " to enable highlighting of `pattern`
let g:haskell_enable_typeroles = 1        " to enable highlighting of type roles
let g:haskell_enable_static_pointers = 1  " to enable highlighting of `static`
let g:haskell_backpack = 1                " to enable highlighting of backpack keywords

let g:haskell_classic_highlighting = 0
let g:haskell_indent_if = 3
let g:haskell_indent_case = 2
let g:haskell_indent_let = 4
let g:haskell_indent_where = 6
let g:haskell_indent_before_where = 2
let g:haskell_indent_after_bare_where = 2
let g:haskell_indent_do = 3
let g:haskell_indent_in = 1
let g:haskell_indent_guard = 2
let g:haskell_indent_case_alternative = 1
let g:cabal_indent_section = 2
" Either check empty($IN_NIX_SHELL) for nix specific or executable('ghcid')
if executable('ghcid')
	let g:ghcid_command = 'ghcid'
else
	let g:ghcid_command = 'stack exec -- ghcid'
endif
if executable('hoogle')
	let g:hoogle_search_bin = 'hoogle'
else
	let g:hoogle_search_bin = 'stack exec -- hoogle'
endif
let g:hoogle_search_count = 30

" Use deoplete
let g:deoplete#enable_at_startup = 1
let g:deoplete#sources = {}
call deoplete#custom#source('_',
            \ 'disabled_syntaxes', ['Comment', 'String'])
call deoplete#custom#option('sources', {
			\ '_' : ['buffer', 'tag', 'around', 'file', 'member'],
			\ 'haskell': ['buffer', 'tag'],
			\ 'rust': ['racer', 'buffer', 'tag'],
			\ 'purescript': ['buffer', 'omni'],
			\})
call deoplete#custom#source('omni', 'functions', {
			\ 'purescript':  'PSCIDEComplete',
			\})
call deoplete#custom#var('omni', 'input_patterns', {
		    \ 'purescript': '\w*',
		    \})
call deoplete#custom#option({
			\ 'auto_complete_delay': 200,
			\ 'auto_complete': v:true,
			\ 'smart_case': v:true,
			\ })
let g:deoplete#sources#rust#disable_keymap = 1
let g:deoplete#sources#rust#racer_binary=expand('$HOME/.cargo/bin/racer')
let g:deoplete#sources#rust#rust_source_path=expand('$HOME/GitSources/rust/src')
let g:rustfmt_autosave = 1

" Use airline
let g:airline#extensions#tabline#enabled = 1
let g:airline#extensions#tabline#fnamemod = ':t'
let g:airline#extensions#tabline#left_sep = ' '
let g:airline#extensions#tabline#left_alt_sep = '|'
let g:airline#extensions#tabline#right_sep = ' '
let g:airline#extensions#tabline#right_alt_sep = '|'
let g:airline#extensions#tabline#formatter = 'default'
let g:airline_left_sep = ' '
let g:airline_left_alt_sep = '|'
let g:airline_right_sep = ' '
let g:airline_right_alt_sep = '|'
let g:airline#extensions#tabline#buffer_nr_show = 1

" Let vim-gitgutter do its thing on large files
let g:gitgutter_max_signs=1000
let g:gitgutter_map_keys = 0
let g:gitgutter_highlight_linenrs = 1
let g:gitgutter_preview_win_floating = 1
let g:gitgutter_use_location_list = 1
let g:gitgutter_sign_added = '+a'
let g:gitgutter_sign_modified = '+m'
let g:gitgutter_sign_removed = '-r'
let g:gitgutter_sign_removed_first_line = '^^'
let g:gitgutter_sign_modified_removed = 'mr'

" Neomake
" compile when I open a file, change text, leave insert mode, etc ...
call neomake#configure#automake({
            \ 'BufWritePost': {},
            \ })
" When compilation is done, open the Location list or quickfix list
" Value of 2 preserves cursor position
let g:neomake_open_list = 2
let g:neomake_warning_sign = {'text': '?'}
let g:neomake_c_enabled_makers = ['gcc']
let g:neomake_c_gcc_maker = {
			\ 'exe': 'gcc',
			\ 'args': ['-Wall', '-Iinclude', '-Wextra', '-Weverything', '-pedantic', '-Wno-sign-conversion'],
			\ }

" Automatically detect style file and apply style to formatting
let g:clang_format#detect_style_file = 1
" Additional trigger not required with deoplete
let g:tmuxcomplete#trigger = ''
" Required for vimtex
let g:vimtex_compiler_progname = 'nvr'
" Fix deoplete's tab behaviour
let g:SuperTabDefaultCompletionType = "<c-n>"
" For workspace
let g:workspace_session_directory = $HOME . '/.vim/session/'
let g:workspace_undodir = $HOME . '/.vim/undodir'
let g:workspace_autosave = 0
let g:workspace_autosave_ignore = ['gitcommit', 'qf', 'tagbar']
let g:workspace_session_disable_on_args = 1
" Toggle quickfix/location list
let g:toggle_list_no_mappings = 1

let g:nvimgdb_config_override = {
  \ 'key_next': 'n',
  \ 'key_step': 's',
  \ 'key_finish': 'f',
  \ 'key_continue': 'c',
  \ 'key_until': 'u',
  \ 'key_breakpoint': 'b',
  \ 'set_tkeymaps': "NvimGdbNoTKeymaps",
  \ }

let g:mapleader = "\<Space>"
let g:maplocalleader = ','
let g:which_key_use_floating_win = 1

let g:cpp_class_scope_highlight = 1
let g:cpp_member_variable_highlight = 1
let g:cpp_class_decl_highlight = 1
let g:cpp_posix_standard = 1
let g:cpp_concepts_highlight = 1

let g:jedi#goto_command = "gt"
let g:jedi#goto_assignments_command = "ga"
let g:jedi#goto_definitions_command = "gd"
let g:jedi#documentation_command = "K"
let g:jedi#usages_command = "gu"
let g:jedi#completions_command = "<Tab>"
let g:jedi#rename_command = "gr"

let g:jedi#auto_initialization = 1
let g:jedi#auto_vim_configuration = 0
let g:jedi#use_tabs_not_buffers = 1
let g:jedi#popup_on_dot = 1
let g:jedi#popup_select_first = 0
let g:jedi#smart_auto_mappings = 0
let g:jedi#show_call_signatures = "2"
let g:jedi#show_call_signatures_delay = 0
let g:jedi#show_call_signatures_modes = 'i'
let g:jedi#enable_speed_debugging = 0
let g:jedi#completions_enabled = 0

" For SLIMV
let g:lisp_rainbow=1
" For Purescript
let g:psc_ide_auto_imports = 1

" FZF
let g:fzf_mru_no_sort = 1
let $FZF_DEFAULT_OPTS='--layout=reverse'
let g:fzf_layout = { 'window': 'call CreateCentredFloatingWindow()' }

" Quickr
let g:quickr_cscope_keymaps = 0

" ----------------------------- Functions ------------------------------------
function! NvimGdbNoTKeymaps()
  tnoremap <silent> <buffer> <Esc> <C-\><C-n>
endfunction

" Stolen from https://github.com/camspiers/dotfiles/blob/master/files/.config/nvim/init.vim
function! CreateCentredFloatingWindow()
	let width = float2nr(&columns * 0.8)
	let height = float2nr(&lines * 0.4)
	let top = ((&lines - height) / 2) - 1
	let left = (&columns - width) / 2
	let opts = {'relative': 'editor', 'row': top, 'col': left, 'width': width, 'height': height, 'style': 'minimal'}

	let top = "╭" . repeat("─", width - 2) . "╮"
	let mid = "│" . repeat(" ", width - 2) . "│"
	let bot = "╰" . repeat("─", width - 2) . "╯"
	let lines = [top] + repeat([mid], height - 2) + [bot]
	let s:buf = nvim_create_buf(v:false, v:true)
	call nvim_buf_set_lines(s:buf, 0, -1, v:true, lines)
	call nvim_open_win(s:buf, v:true, opts)
	set winhl=Normal:Floating
	let opts.row += 1
	let opts.height -= 2
	let opts.col += 2
	let opts.width -= 4
	call nvim_open_win(nvim_create_buf(v:false, v:true), v:true, opts)
	au BufWipeout <buffer> exe 'bw '.s:buf
endfunction

function! s:incsearch_config(...) abort
	return incsearch#util#deepextend(deepcopy({
				\   'modules': [incsearch#config#easymotion#module({'overwin': 1})],
				\   'keymap': {
				\     "\<CR>": '<Over>(easymotion)'
				\   },
				\   'is_expr': 0
				\ }), get(a:, 1, {}))
endfunction
